@@ -1254,21 +1254,18 @@ stop_cb (GtkAction *action,
 	GSRWindowPrivate *priv = window->priv;
 
 	/* Work out what's playing */
-	if (priv->play && priv->play->state == GST_STATE_PLAYING) {
+	if (priv->play && priv->play->state >= GST_STATE_PAUSED) {
+		GST_DEBUG ("Stopping play pipeline");
 		set_pipeline_state_to_null (priv->play->pipeline);
 	} else if (priv->record && priv->record->state == GST_STATE_PLAYING) {
-		GstPad *pad;
+		GST_DEBUG ("Stopping recording source");
+		/* GstBaseSrc will automatically send an EOS when stopping */
+		gst_element_set_state (priv->record->src, GST_STATE_NULL);
+		gst_element_get_state (priv->record->src, NULL, NULL, -1);
+		gst_element_set_locked_state (priv->record->src, TRUE);
 
-		pad = gst_element_get_pad (priv->record->src, "src");
-		if (pad) {
-			/* FIXME: this is a bit evil, we should have a clean way of
-			 * doing this programmatically. Or maybe GstBaseSrc should just
-			 * send an EOS itself in PAUSED => READY? */
-			GST_PAD_STREAM_LOCK (pad);
-			gst_pad_push_event (pad, gst_event_new_eos ());
-			GST_PAD_STREAM_UNLOCK (pad);
-			gst_object_unref (pad);
-		}
+		GST_DEBUG ("Stopping recording pipeline");
+		set_pipeline_state_to_null (priv->record->pipeline);
 	}
 }
 
@@ -1722,11 +1719,13 @@ record_state_changed_cb (GstBus *bus, GstMessage *msg, GSRWindow *window)
 	case GST_STATE_PLAYING:
 		window->priv->record_id = g_idle_add (record_start, window);
 		gtk_widget_set_sensitive (window->priv->profile, FALSE);
+		gtk_widget_set_sensitive (window->priv->input, FALSE);
 		break;
 	case GST_STATE_READY:
 		gtk_adjustment_set_value (GTK_RANGE (window->priv->scale)->adjustment, 0.0);
 		gtk_widget_set_sensitive (window->priv->scale, FALSE);
 		gtk_widget_set_sensitive (window->priv->profile, TRUE);
+		gtk_widget_set_sensitive (window->priv->input, TRUE);
 		/* fall through */
 	case GST_STATE_PAUSED:
 		set_action_sensitive (window, "Stop", FALSE);
@@ -1747,79 +1746,6 @@ record_state_changed_cb (GstBus *bus, GstMessage *msg, GSRWindow *window)
 	}
 }
 
-/* go through a bin, finding the one pad that is unconnected in the given
- * direction, and return that pad; nicked from the GStreamer gconf plugin */
-static GstPad *
-notgst_bin_find_unconnected_pad (GstBin * bin, GstPadDirection direction)
-{
-	GstPad *pad = NULL;
-	GList *elements = NULL;
-	const GList *pads = NULL;
-	GstElement *element = NULL;
-
-	GST_OBJECT_LOCK (bin);
-	elements = bin->children;
-	/* traverse all elements looking for unconnected pads */
-	while (elements && pad == NULL) {
-		element = GST_ELEMENT (elements->data);
-		GST_OBJECT_LOCK (element);
-		pads = element->pads;
-		while (pads) {
-			GstPad *testpad = GST_PAD (pads->data);
-
-			/* check if the direction matches */
-			if (GST_PAD_DIRECTION (testpad) == direction) {
-				GST_OBJECT_LOCK (testpad);
-				if (GST_PAD_PEER (testpad) == NULL) {
-					GST_OBJECT_UNLOCK (testpad);
-					/* found it ! */
-					pad = testpad;
-					break;
-				}
-				GST_OBJECT_UNLOCK (testpad);
-			}
-			pads = g_list_next (pads);
-		}
-		GST_OBJECT_UNLOCK (element);
-		elements = g_list_next (elements);
-	}
-	GST_OBJECT_UNLOCK (bin);
-
-	return pad;
-}
-
-/* nicked from the GStreamer gconf plugin */
-static GstElement *
-notgst_render_bin_from_description (const gchar * description, GError ** err)
-{
-	GstPad *pad = NULL;
-	GstBin *bin;
- 	gchar *desc;
-
-	GST_DEBUG ("making bin from description '%s'", GST_STR_NULL (description));
-
- 	/* parse the pipeline to a bin */
- 	desc = g_strdup_printf ("bin.( %s )", description);
-	bin = (GstBin *) gst_parse_launch (desc, err);
-	g_free (desc);
-
-	if (bin == NULL || (err && *err != NULL)) {
-		if (bin)
-			gst_object_unref (bin);
-		return NULL;
-	}
-
-	/* find pads and ghost them if necessary */
-	if ((pad = notgst_bin_find_unconnected_pad (bin, GST_PAD_SRC))) {
-		gst_element_add_pad (GST_ELEMENT (bin), gst_ghost_pad_new ("src", pad));
-	}
-	if ((pad = notgst_bin_find_unconnected_pad (bin, GST_PAD_SINK))) {
-		gst_element_add_pad (GST_ELEMENT (bin), gst_ghost_pad_new ("sink", pad));
-	}
-
-	return GST_ELEMENT (bin);
-}
-
 /* create the gconf-based source for recording.
  * store the source and the mixer in it in our window-private data
  */
@@ -1947,7 +1873,8 @@ make_record_pipeline (GSRWindow *window)
 		GST_STR_NULL (profile_pipeline_desc));
 
 	pipeline_desc = g_strdup_printf ("audioconvert ! %s", profile_pipeline_desc);
-	encoder = notgst_render_bin_from_description (pipeline_desc, &err);
+	GST_DEBUG ("making encoder bin from description '%s'", pipeline_desc);
+	encoder = gst_parse_bin_from_description (pipeline_desc, TRUE, &err);
 	g_free (pipeline_desc);
 	pipeline_desc = NULL;
 
